//
//  **************************************************************************
//
//  dialog.cpp
//  (C) 2003 Bosco K. Ho 
//
//  **************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  **************************************************************************
//
#include "dialog.h"
#include "wx/wx.h"
#include "event.h"

BEGIN_EVENT_TABLE(TorsionDialog, wxDialog)
  EVT_BUTTON(wxID_OK, TorsionDialog::OnOk)
END_EVENT_TABLE()



TorsionDialog::TorsionDialog(wxWindow *parent, 
                             Controller* pController):
   wxDialog(parent, -1, _T(""), wxDefaultPosition),
   mpController(pController)
{
  wxString text;
  MyProtein* p_protein = mpController->mpProtein;
  
  SetTitle(_T("Set backbone"));
  wxBoxSizer* top_sizer = new wxBoxSizer(wxVERTICAL);

  // introductory text 
  wxString intro; 
  double init_phi = 0.0;
  double init_psi = 0.0;
  wxString res;

  int nSelected = p_protein->mSelect.nSelected();

  if (nSelected == 0)
  {
    intro = _T("No residues selected.");
    top_sizer->Add(new wxStaticText(this, -1, intro), 0, wxALL, 10);
  }
  else
  {
    intro = _T("Set backbone torsions of \n");
    
    int n_counted = 0;
    for (int i=0; i<p_protein->nRes(); i++)
    {
      if (p_protein->mSelect.isSelected(i))
      {
        if (n_counted > 45)
        {
          intro += " ...";
          break;
        }
        else if (n_counted > 0)
          intro += ", ";

        if ( (n_counted%6) == 0 )
          intro += "\n";

        res.Printf(_T("%s%d"), p_protein->resName(i).c_str(), p_protein->resNo(i));
        intro += res;

        init_phi += p_protein->phi(i);
        init_psi += p_protein->psi(i);
        
        n_counted += 1;
      }
    }
    init_phi /= double(nSelected);
    init_psi /= double(nSelected);
    
    intro += _T("\n\nRange: -180 to 180 degrees");
    top_sizer->Add(new wxStaticText(this, -1, intro), 0, wxALL, 10);


    // make phi_sizer for phi torsion control
    wxBoxSizer* phi_sizer = new wxBoxSizer(wxHORIZONTAL);
  
    phi_sizer->Add(new wxStaticText(this, -1, _T("PHI ")), 1, wxALL, 2);
  
    text.Printf(_T("%.f"), init_phi);
    mpPhiSpinCtrl = new wxSpinCtrl(this, -1, text, wxDefaultPosition, 
                                   wxSize(60,-1), wxSP_ARROW_KEYS |wxSP_WRAP);
    mpPhiSpinCtrl->SetRange(-180, 180);
    phi_sizer->Add(mpPhiSpinCtrl, 0, wxALL, 2);
  
  
    // make psi_sizer for psi torsion control
    wxBoxSizer* psi_sizer = new wxBoxSizer(wxHORIZONTAL);
  
    psi_sizer->Add(new wxStaticText(this, -1, _T("PSI ")), 1, wxALL, 2);
    text.Printf(_T("%.f"), init_psi);
    mpPsiSpinCtrl = new wxSpinCtrl(this, -1, text, wxDefaultPosition, 
                                   wxSize(60, -1), wxSP_ARROW_KEYS |wxSP_WRAP);
    mpPsiSpinCtrl->SetRange(-180, 180);
    psi_sizer->Add(mpPsiSpinCtrl, 0, wxALL, 2);

    
    wxBoxSizer* torsion_sizer = new wxBoxSizer(wxVERTICAL);
    torsion_sizer->Add(phi_sizer, 1, wxALL, 0);
    torsion_sizer->Add(psi_sizer, 1, wxALL, 0);
    top_sizer->Add(torsion_sizer, 1, wxALL, 10);
  }
  
  // Add ok and cancel buttons
  wxBoxSizer* button_sizer = new wxBoxSizer( wxHORIZONTAL );
  button_sizer->Add(new wxButton(this, wxID_OK, "OK"), 0, wxALL, 2);
  button_sizer->Add(new wxButton(this, wxID_CANCEL, "Cancel"), 0, wxALL, 2);

  top_sizer->Add(button_sizer, 1, wxALL, 10);

  SetAutoLayout(TRUE);
  SetSizer(top_sizer);

  top_sizer->Fit(this);
  top_sizer->SetSizeHints(this);
}




void TorsionDialog::OnOk(wxCommandEvent& WXUNUSED(event))
{

  if (mpController->mpProtein->mSelect.nSelected() > 0)
  {
  
    double phi = (double) mpPhiSpinCtrl->GetValue();
    double psi = (double) mpPsiSpinCtrl->GetValue();
  
    mpController->setPhiPsi(phi, psi);
    mpController->setUndoMarker("backbone torsion");
    mpController->countInteractions();
  }
   
  Close();
}





/////////////////////////////////////////////////
//  PopupMenu
/////////////////////////////////////////////////

void PopupMenu::show(int x, int y)
{
  wxMenu menu(_T("Edit Selection"));

  menu.Append(EVENT_CLEAR_SELECTION, _T("&Clear Selection"));
  menu.Append(EVENT_SELECT_ALL, _T("Select All"));

  menu.AppendSeparator();
  
  menu.Append(EVENT_COPY_TO_PDB, _T("Copy to PDB"));
  menu.Append(EVENT_INSERT_FROM_PDB, _T("Insert from PDB"));
  menu.Append(EVENT_INSERT_ALA, _T("Insert ALA"));

  menu.AppendSeparator();

  menu.Append(EVENT_CUT, _T("Cut"));
  menu.Append(EVENT_COPY, _T("Copy"));
  menu.Append(EVENT_DELETE, _T("Delete"));
  menu.Append(EVENT_INSERT, _T("Paste"));
  menu.Append(EVENT_INSERT_RIGHT, _T("Paste to right"));

  menu.AppendSeparator();
  
  menu.Append(EVENT_BACKBONE, _T("Change &Backbone"));

  menu.AppendSeparator();
  
  menu.Append(EVENT_MUTATE_ALA, _T("Mutate to ALA")); 
  menu.Append(EVENT_MUTATE_ARG, _T("Mutate to ARG")); 
  menu.Append(EVENT_MUTATE_ASN, _T("Mutate to ASN")); 
  menu.Append(EVENT_MUTATE_ASP, _T("Mutate to ASP")); 
  menu.Append(EVENT_MUTATE_CYS, _T("Mutate to CYS")); 
  menu.Append(EVENT_MUTATE_GLN, _T("Mutate to GLN")); 
  menu.Append(EVENT_MUTATE_GLU, _T("Mutate to GLU")); 
  menu.Append(EVENT_MUTATE_GLY, _T("Mutate to GLY")); 
  menu.Append(EVENT_MUTATE_HIS, _T("Mutate to HIS")); 
  menu.Append(EVENT_MUTATE_ILE, _T("Mutate to ILE")); 
  menu.Append(EVENT_MUTATE_LEU, _T("Mutate to LEU")); 
  menu.Append(EVENT_MUTATE_LYS, _T("Mutate to LYS")); 
  menu.Append(EVENT_MUTATE_MET, _T("Mutate to MET")); 
  menu.Append(EVENT_MUTATE_PHE, _T("Mutate to PHE")); 
  menu.Append(EVENT_MUTATE_PRO, _T("Mutate to PRO")); 
  menu.Append(EVENT_MUTATE_SER, _T("Mutate to SER")); 
  menu.Append(EVENT_MUTATE_THR, _T("Mutate to THR")); 
  menu.Append(EVENT_MUTATE_TRP, _T("Mutate to TRP")); 
  menu.Append(EVENT_MUTATE_TYR, _T("Mutate to TYR")); 
  menu.Append(EVENT_MUTATE_VAL, _T("Mutate to VAL")); 

  mpParent->PopupMenu(&menu, x, y);
}








