//
//  ****************************************************************************
//
//  graphic3d.h
//
//  (C) Thu Mar 25 2004 Bosco K. Ho 
//
//  ****************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  ****************************************************************************
//
//

#ifndef _graphic3d_h_
#define _graphic3d_h_

#include <list>
#include "vector3d.h"
#include "wx/dcbuffer.h"

// color indices
#define WHITE         255
#define LIGHT_GREY    210
#define GREY          180
#define DARK_GREY     160
#define DARKEST_GREY  140
#define BLACK         128
#define RED           1
#define BLUE          2
#define YELLOW        3
#define GREEN         4
#define DARK_GREEN    6
#define LIGHT_BLUE    7
#define ORANGE        8
#define LIGHT_ORANGE  9
#define PURPLE        10



class GfxObject 
{
public:
  GfxObject(): id(-1), z(0), width(0), color(0), pString(NULL) 
    { for (int i=0; i<8; i++) v[i] = 0; }
  int id;
  int v[8];
  int z;
  int width;
  int color;
  std::string* pString;
};

inline bool operator < (const GfxObject& a, const GfxObject& b)  { return (a.z > b.z);  }

// must use list for the fastest sorting
typedef std::list<GfxObject>    GfxObjectList;

typedef GfxObjectList::iterator GfxObjectIter;
 


class Project3d
{
  public:
    Project3d() { initColorTable(); }
    ~Project3d() {}

    void initColorTable();
    void clear();

    Vector3d point(Vector3d p);

    void line(Vector3d& p1, Vector3d& p2, int color, double width);
    void dashLine(Vector3d& p1, Vector3d& p2, int color, double width);
    void solidLine(Vector3d& p1, Vector3d& p2, int color, double width);
    void quad(Vector3d v[], int color, double width);
    void solidQuad(Vector3d v[], int color);
    void ring(Vector3d& p, double r, double width, int color);
    void solidSphere(Vector3d& p, double r, int color);
    void string(Vector3d& p, std::string* pString, int color);
    
    void setCamera(Vector3d& pos, Vector3d& front, Vector3d& up);
    void setViewport(int x, int y, int w, int h);

    bool isXYOverSphere(Vector3d& v, double r, int screenX, int screenY, int* pZOut);

    // wxWidget specific routines
    void drawZ(wxBufferedPaintDC &dc); 
    void drawZFixLight(wxBufferedPaintDC &dc, Vector3d lightCenter); 
    void drawZLight(wxBufferedPaintDC &dc); 
    void drawZFlat(wxBufferedPaintDC &dc); 
    
    wxColor getZColor(int index, int z); 

    Vector3d mCameraUp;
    Vector3d mCameraPos;
    Vector3d mCameraFront;

  private:
    int mXOffset;
    int mYOffset;
   
    double mPerspXScale;
    double mPerspYScale;
    
    double mMinZ;
    double mDeltaZ;
     
    GfxObjectList mGfxObjectList;
    Matrix3d mProjectMatrix;
    wxColor mColorTable[256];
};

#endif
