//
//  ***************************************************************************
//
//  loop.cpp
//  (C) 2004 Bosco K. Ho 
//
//
//  ***************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  ****************************************************************************
//

#ifndef _loop_h_
#define _loop_h_

#include "protein.h"
#ifdef _MSC_VER
  extern "C" {
    #include "closure.h"
  }
#else
    #include "closure.h"
#endif

typedef std::vector<Protein*> LoopSolns;

class LoopClosure
{
public:
  LoopClosure(): mpProtein(NULL), miSolution(0) {};
  ~LoopClosure() { clearSolns(mpSolnLoops); };

  void setSolution(int i);
  int nSolution() { return (int) mpSolnLoops.size(); }
  void drive(int res, double phi, double psi);
  int iSolution() { return miSolution; }
  void setLoop(Protein* p_protein, int res1, int res2, int res3);
  void clearSolns(LoopSolns& solns);
  void clear();
  void copySolns(LoopSolns& saved_copy);
  void restoreSolns(LoopSolns& saved_copy);
  void toggleRes(int res);
  int nSelectedRes();
  bool isSelectedRes(int res);
  int res(int i);
  void copyLoopToProtein(Protein* p_loop, int res);

private:
  void calculateSolutions();
  bool hasProlineViolation(Protein* p_loop);
  int iSolutionClosestToAtom(int res, int atom_key);
  void rotateLoopToSoln(Protein* p_loop, 
                       Vector3d new_ca2, Vector3d new_c1, Vector3d new_c2);
  void extractParameters(Protein* p_loop, int res1, int res2, int res3);
  void copySidechain(Protein* p_sidechain, int sc_res, Protein* p_target, int tar_res);

  int mRes1, mRes2, mRes3;
  Protein* mpProtein;

  int mLoopRes1, mLoopRes2, mLoopRes3;
  Protein mLoop;

  int miSolution;
  LoopSolns mpSolnLoops;
};


#endif

