//
//  **************************************************************************
//
//  residue.cpp
//
//  (C) 2003 Bosco K. Ho 
//
//  **************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  **************************************************************************
//

#include <string>
#include "residue.h"

using namespace std;



// data structure to store information for access functions
class ResTypeInfoBlk 
{
  public:
    ResTypeInfoBlk(const char neighbour, std::string _ch3, std::string mName) :
      name1(neighbour), name3(_ch3), name(mName) {}
    char         name1;
    std::string  name3;
    std::string  name;
};



static const ResTypeInfoBlk resTypeInfo[] = 
{
  ResTypeInfoBlk('X', "XXX", "unknown"          ),
  ResTypeInfoBlk('A', "ALA", "alanine"          ),
  ResTypeInfoBlk('C', "CYS", "cystine"          ),
  ResTypeInfoBlk('D', "ASP", "aspartic acid"    ),
  ResTypeInfoBlk('E', "GLU", "glutamatic acid"  ),
  ResTypeInfoBlk('F', "PHE", "phenlyalanine"    ),
  ResTypeInfoBlk('G', "GLY", "glycine"          ),
  ResTypeInfoBlk('H', "HIS", "histidine"        ),
  ResTypeInfoBlk('I', "ILE", "isoleucine"       ),  
  ResTypeInfoBlk('K', "LYS", "lysine"           ), 
  ResTypeInfoBlk('L', "LEU", "leucine"          ),
  ResTypeInfoBlk('M', "MET", "methionine"       ),   
  ResTypeInfoBlk('N', "ASN", "asparagine"       ),  
  ResTypeInfoBlk('P', "PRO", "proline"          ),
  ResTypeInfoBlk('Q', "GLN", "glutamine"        ),
  ResTypeInfoBlk('R', "ARG", "arginine"         ),
  ResTypeInfoBlk('S', "SER", "serine"           ),  
  ResTypeInfoBlk('T', "THR", "threonine"        ),
  ResTypeInfoBlk('V', "VAL", "valine"           ), 
  ResTypeInfoBlk('W', "TRP", "tryptophan"       ),
  ResTypeInfoBlk('Y', "TYR", "tyrosine"         ),  
  ResTypeInfoBlk('>', "ACE", "acetyl"           ),  
  ResTypeInfoBlk('<', "NME", "methyl"           )  
};  



// parses an atom name string into the atom key
int get_atom_key(string s)
{
  if (s.length() < 4) return XX;

  if (s == " H1 ") return H1;
  if (s == " H2 ") return H2;
  if (s == " H3 ") return H3;

  if (s[3]==' ') 
    s[3]='1';

  if ((s[1]=='H') && (s[0]==' ')) 
    s[0]='1';

 
  if (s == " CH3") return CA;
  // can do this as there are no name clashes
  if (s == "1HH3") return H1;
  if (s == "2HH3") return H2;
  if (s == "3HH3") return H3;

  if (s == " CA1") return CA;
  if (s == " C 1") return C ;
  if (s == "1HA1") return HA1;
  if (s == "2HA1") return HA2;
  if (s == " N 1") return N ;
  if (s == " O 1") return O ;
  if (s == "1H 1") return H ;
  if (s == "2H 1") return H2;
  if (s == "3H 1") return H3;
  if (s == " OXT") return OXT ;
  if (s == " CB1") return CB ;
  if (s == "1HB1") return HB1;
  if (s == "2HB1") return HB2;
  if (s == "3HB1") return HB3;
  if (s == " CG2") return CG2 ;
  if (s == "1HG2") return HG21;
  if (s == "2HG2") return HG22;
  if (s == "3HG2") return HG23;
  if (s == " SG1") return SG1 ;
  if (s == " CG1") return CG1 ;
  if (s == " OG1") return OG1 ;
  if (s == "1HG1") return HG11;
  if (s == "2HG1") return HG12;
  if (s == "3HG1") return HG13;
  if (s == " SD1") return SD1 ;
  if (s == " CD1") return CD1 ;
  if (s == " CD2") return CD2 ;
  if (s == " ND1") return ND1 ;
  if (s == " ND2") return ND2 ;
  if (s == " OD1") return OD1 ;
  if (s == " OD2") return OD2 ;
  if (s == "1HD1") return HD11;
  if (s == "2HD1") return HD12;
  if (s == "3HD1") return HD13;
  if (s == "1HD2") return HD21;
  if (s == "2HD2") return HD22;
  if (s == "3HD2") return HD23;
  if (s == " CE1") return CE1 ;
  if (s == " CE2") return CE2 ;
  if (s == " CE3") return CE3 ;
  if (s == " OE1") return OE1 ;
  if (s == " OE2") return OE2 ;
  if (s == " NE1") return NE1 ;
  if (s == " NE2") return NE2 ;
  if (s == "1HE1") return HE11;
  if (s == "2HE1") return HE12;
  if (s == "3HE1") return HE13;
  if (s == "1HE2") return HE21;
  if (s == "2HE2") return HE22;
  if (s == "3HE2") return HE23;
  if (s == "1HE3") return HE31;
  if (s == " CZ1") return CZ1 ;
  if (s == " CZ2") return CZ2 ;
  if (s == " CZ3") return CZ3 ;
  if (s == " NZ1") return NZ1 ;
  if (s == "1HZ1") return HZ11;
  if (s == "2HZ1") return HZ12;
  if (s == "3HZ1") return HZ13;
  if (s == "1HZ2") return HZ21;
  if (s == "1HZ3") return HZ31;
  if (s == " CH2") return CH2 ;
  if (s == " OH1") return OH1 ;
  if (s == " NH1") return NH1 ;
  if (s == " NH2") return NH2 ;
  if (s == "1HH1") return HH11;
  if (s == "2HH1") return HH12;
  if (s == "1HH2") return HH21;
  if (s == "2HH2") return HH22;
  
  cerr << "Warning: atom not parsed: " << s << endl;
  return XX;
}


string get_atom_name(int resKey, int atomKey)
{
  string s;
  switch (atomKey) 
  {
    case CA  : 
          if ((resKey == ACE)||(resKey == NME))
            s = " CH3"; 
          else
            s = " CA "; 
          break;
    case CB  : s = " CB "; break;
    case HA1 :  
          if (resKey == GLY) 
               s = "1HA "; 
          else 
               s = " HA "; 
          break;
    case HA2 : s = "2HA "; break;
    case H   : s = " H  "; break;
    case H1  : 
             if ((resKey == ACE)||(resKey == NME))
               s = "1HH3"; 
             else
               s = "1H  "; 
             break;
    case H2  :
             if ((resKey == ACE)||(resKey == NME))
               s = "2HH3"; 
             else
               s = "2H  "; 
             break;
    case H3  :
             if ((resKey == ACE)||(resKey == NME))
               s = "3HH3"; 
             else
               s = "3H  "; 
             break;
    case OXT : s = " OXT"; break;
    case O   : s = " O  "; break;
    case N   : s = " N  "; break;
    case C   : s = " C  "; break;
    case HB1 : 
             if ((resKey == VAL)||(resKey == THR)||(resKey == ILE))
               s = " HB "; 
             else
               s = "1HB "; 
             break;
    case HB2 : s = "2HB "; break;
    case HB3 : s = "3HB "; break;
    case CG2 : s = " CG2"; break;
    case HG21: s = "1HG2"; break;
    case HG22: s = "2HG2"; break;
    case HG23: s = "3HG2"; break;
    case SG1 : s = " SG "; break;
    case CG1 : 
             if ((resKey == VAL)||(resKey == ILE)||(resKey == THR))
               s = " CG1"; 
             else
               s = " CG "; 
             break;
    case OG1 : 
             if ((resKey == SER))
               s = " OG "; 
             else // THR
               s = " OG1"; 
             break;
    case HG11: 
             if (resKey == THR)
               s = " HG1"; 
             else if ((resKey == SER)||(resKey == LEU))
               s = " HG "; 
             else if ((resKey == VAL)||(resKey == ILE))
               s = "1HG1";
             else
               s = "1HG "; 
             break;
    case HG12:
             if ((resKey == VAL)||(resKey == ILE))
               s = "2HG1";
             else
               s = "2HG "; 
             break;
    case HG13: s = "3HG1"; break;
    case SD1 : s = " SD "; break;
    case CD1 : 
             if ( (resKey == LEU) || (resKey == PHE)|| (resKey == HIS)
               || (resKey == TYR) || (resKey == TRP) )
               s = " CD1"; 
             else
               s = " CD "; 
             break;
    case CD2 : s = " CD2"; break;
    case ND1 : s = " ND1"; break;
    case ND2 : s = " ND2"; break;
    case OD1 : s = " OD1"; break;
    case OD2 : s = " OD2"; break;

    case HD11: 
             if ((resKey == ASP)||(resKey == PHE)||(resKey == TYR)||(resKey == TRP)||(resKey == HIS))
               s = " HD1"; 
             else if ((resKey == LEU)||(resKey == ILE))
               s = "1HD1"; 
             else
               s = "1HD "; 
            break;
    case HD12: 
             if ((resKey == LEU)||(resKey == ILE))
              s = "2HD1"; 
            else
              s = "2HD "; 
            break;
    case HD13: s = "3HD1"; break;

    case HD21: 
            if ((resKey == PHE)||(resKey == TYR)||(resKey == TRP)||(resKey == HIS))
              s = " HD2"; 
            else // LEU & ASN
              s = "1HD2"; 
            break;
    case HD22: s = "2HD2"; break;
    case HD23: s = "3HD2"; break;
    case CE1 : 
             if ((resKey == HIS)||(resKey == PHE)||(resKey == TYR)||(resKey == TRP))
               s = " CE1"; 
             else
               s = " CE "; 
             break;
    case CE2 : s = " CE2"; break;
    case CE3 : s = " CE3"; break;
    case OE1 : s = " OE1"; break;
    case OE2 : s = " OE2"; break;
    case NE1 : 
             if ((resKey == TRP))
               s = " NE1"; 
             else
               s = " NE "; 
             break;
    case NE2 : s = " NE2"; break;

    case HE11: 
             if ((resKey == GLU)||(resKey == PHE)||(resKey == TYR)||(resKey == TRP)||(resKey == HIS))
               s = " HE1"; 
             else if ((resKey == HIS)||(resKey == ARG))
               s = " HE "; 
             else if (resKey == MET)
               s = "1HE "; 
             else // LYS
               s = " HE1"; 
            break;
    case HE12: s = "2HE "; break;
    case HE13: s = "3HE "; break;

    case HE21: 
             if ((resKey == HIS)||(resKey == PHE)||(resKey == TYR)||(resKey == TRP))
               s = " HE2"; 
             else if (resKey == GLN)
               s = "1HE2"; 
             else // LYS
               s = "1HE "; 
            break;
    case HE22: 
             if (resKey == GLN)
               s = "2HE2"; 
             else // LYS
               s = "2HE "; 
            break;
    case HE23: s = "3HE2"; break;
    case HE31: s = " HE3"; break;
    case CZ1 : s = " CZ "; break;
    case CZ2 : s = " CZ2"; break;
    case CZ3 : s = " CZ3"; break;
    case NZ1 : s = " NZ "; break;

    case HZ11: 
             if ((resKey == PHE))
               s = " HZ "; 
             else // LYS
               s = "1HZ "; 
            break;
    case HZ12: s = "2HZ "; break;
    case HZ13: s = "3HZ "; break;

    case HZ21: s = " HZ2"; break;
    case HZ31: s = " HZ3"; break;
    case CH2 : s = " CH2"; break;
    case OH1 : s = " OH "; break;
    case NH1 : s = " NH1"; break;
    case NH2 : s = " NH2"; break;
    case HH11: 
             if (resKey == TYR)
               s = " HH "; 
             else
               s = "1HH1"; 
             break;
    case HH12: s = "2HH1"; break;
    case HH21: 
             if (resKey == TRP)
               s = " HH2"; 
             else // ARG
               s = "1HH2"; 
             break;
    case HH22: s = "2HH2"; break;
    default:   s = "    ";
  }

  return s;
}


int get_res_key(const std::string& s) 
{
  if (s.length() == 1) 
  {
    for (int i=1; i<=N_RES_TYPE; i++)
      if (s[0] == resTypeInfo[i].name1)  return i;
  }
  else if (s.length() == 3) 
  {
    for (int i=1; i<=N_RES_TYPE; i++)
      if (s == resTypeInfo[i].name3)  return i;

    // special histidine names for amber and charmm
    if ((s == "HSE") || (s == "HSD") || (s == "HID") 
      || (s == "HIE")  || (s == "HIP"))
      return HIS;
  }
  else 
  {
    for (int i=1; i<=N_RES_TYPE; i++)
      if (s == resTypeInfo[i].name)  return i;
  }
  cerr << "Warning: residue not parsed: " << s << endl;
  return XXX;
}



const std::string& get_res_name_3(int key) 
{
  return resTypeInfo[key].name3;
}



const char get_res_char(int key) 
{
  return resTypeInfo[key].name1;
}



const std::string& get_res_name(int key) 
{
  return resTypeInfo[key].name;
}



int get_n_chi(int resKey) 
{
  for (int nChi = 0; nChi < 5; nChi++)
    if (res_chi_tetrad [resKey][nChi][0] == XX) return nChi;
  return 0;  
}







