//
//  ***************************************************************************
//
//  superpose.h
//  (C) 2004 Bosco K. Ho 
//
//  An object that calculates the optimal rotation between two sets of points
//  and calculates the rmsd. It is a C++ wrapper around the c files
//  rmsd.c and rmsd.h.
//
//  ***************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  ****************************************************************************
//

#ifndef _SUPERPOSE_H_
#define _SUPERPOSE_H_

#include <vector>
#include "vector3d.h"

class Superpose
{
public:
  void addPoint(Vector3d* pRef, Vector3d* pMov);
  void clear(); 
  void calculateOptimalRotation();
  Matrix3d matrix() { return mMatrix; };  
  double rmsd() { return mRmsd; };
  double nPoint() { return mRefList.size(); }
  
private:
  typedef std::vector<Vector3d*> PVecList;
  PVecList mRefList, mMovList;
  double mRmsd;
  Matrix3d mMatrix;
};

#endif
